@@ -2,9 +2,9 @@
  *  this file is part of wdfs --> http://noedler.de/projekte/wdfs/
  *
  *  wdfs is a webdav filesystem with special features for accessing subversion
- *  repositories. it is based on fuse v2.3+ and neon v0.24.7+.
+ *  repositories. it is based on fuse v2.5+ and neon v0.24.7+.
  * 
- *  copyright (c) 2005 - 2006 jens m. noedler, noedler@web.de
+ *  copyright (c) 2005 - 2007 jens m. noedler, noedler@web.de
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <assert.h>
+#include <termios.h>
 #include <ne_basic.h>
 #include <ne_auth.h>
 #include <ne_locks.h>
@@ -49,13 +51,69 @@ ne_lock_store *store = NULL;
 struct ne_auth_data auth_data;
 
 
-static int ne_set_server_auth_callback(
-	void *userdata, const char *realm, int attempt, char *user, char *password)
+/* reads from the terminal without displaying the typed chars. used to type
+ * the password savely. */
+static int fgets_hidden(char *lineptr, size_t n, FILE *stream)
 {
-	assert(auth_data.username && auth_data.password);
+	struct termios settings_old, settings_new;
+	int ret = 0;
 
-	strncpy(user, 		auth_data.username, NE_ABUFSIZ);
-	strncpy(password, 	auth_data.password, NE_ABUFSIZ);
+	if (isatty(fileno(stream)))	{
+		/* turn echoing off and fail if we can't */
+		if (tcgetattr(fileno(stream), &settings_old) == 0) {
+			settings_new = settings_old;
+			settings_new.c_lflag &= ~ECHO;
+			if (tcsetattr(fileno(stream), TCSAFLUSH, &settings_new) == 0)
+				ret = 1;
+		}
+	}
+	else
+		ret = 1;
+
+	/* read the password */
+	if (ret == 1 &&	fgets(lineptr, n, stream) == NULL)
+		ret = 0;
+
+	if (isatty(fileno(stream)))
+		/* restore terminal */
+		(void) tcsetattr(fileno(stream), TCSAFLUSH, &settings_old);
+
+	return ret;
+}
+
+
+/* this method is envoked, if the server requires authentication */
+static int ne_set_server_auth_callback(
+	void *userdata, const char *realm, 
+	int attempt, char *username, char *password)
+{
+	const size_t length = 100;
+	char buffer[length];
+
+	/* ask the user for the username and password if needed */
+	if (auth_data.username == NULL) {
+		printf("username: ");
+		if (fgets(buffer, length, stdin)) {
+			int len = strlen(buffer);
+			if (buffer[len - 1] == '\n')
+				buffer[len - 1] = '\0';
+			auth_data.username = strdup(buffer);
+		}
+	}
+	if (auth_data.password == NULL) {
+		printf("password: ");
+		if (fgets_hidden(buffer, length, stdin)) {
+			int len = strlen(buffer);
+			if (buffer[len - 1] == '\n')
+				buffer[len - 1] = '\0';
+			auth_data.password = strdup(buffer);
+		}
+		printf("\n");
+	}
+
+	assert(auth_data.username && auth_data.password);
+	strncpy(username, auth_data.username, NE_ABUFSIZ);
+	strncpy(password, auth_data.password, NE_ABUFSIZ);
 
 	return attempt;
 }
@@ -73,20 +131,21 @@ static int verify_ssl_certificate(
 	ident = ne_ssl_cert_identity(certificate);
 
 	if (ident) {
-		printf("WARNING: untrusted server certificate for '%s':\n", ident);
+		fprintf(stderr,
+			"WARNING: untrusted server certificate for '%s':\n", ident);
 	}
 
 	if (failures & NE_SSL_IDMISMATCH) {
-		printf(" certificate was issued to hostname '%s' rather than '%s'\n", 
+		fprintf(stderr,
+			" certificate was issued to hostname '%s' rather than '%s'\n", 
 			ne_ssl_cert_identity(certificate), uri->host);
-		printf(" this connection could have been intercepted!\n");
+		fprintf(stderr, " this connection could have been intercepted!\n");
 	}
 
 	ne_ssl_cert_validity(certificate, from, to);
-	printf(" certificate is valid from %s to %s", from, to);
+	printf(" certificate is valid from %s to %s\n", from, to);
 	if (failures & NE_SSL_EXPIRED)
-		printf(" >> certificate expired! <<");
-	printf("\n");
+		fprintf(stderr, " >> certificate expired! <<\n");
 
 	char *issued_to = ne_ssl_readable_dname(ne_ssl_cert_subject(certificate));
 	char *issued_by = ne_ssl_readable_dname(ne_ssl_cert_issuer(certificate));
@@ -95,7 +154,7 @@ static int verify_ssl_certificate(
 	free_chars(&issued_to, &issued_by, NULL);
 
 	/* don't prompt the user if the parameter "-ac" was passed to wdfs */
-	if (accept_certificate == true)
+	if (wdfs.accept_certificate == true)
 		return 0;
 
 	/* prompt the user wether he/she wants to accept this certificate */
@@ -120,34 +179,22 @@ static int verify_ssl_certificate(
 }
 
 
-/* sets up a webdav connection. if you want to use authentication, pass username
- * and password. if not pass twise NULL. returns 0 on success or -1 on error. */
+/* sets up a webdav connection. if the servers needs authentication, the passed
+ * parameters username and password are used. if they were not passed they can
+ * be entered interactively. this method returns 0 on success or -1 on error. */
 int setup_webdav_session(
 	const char *uri_string, const char *username, const char *password)
 {
 	assert(uri_string);
 
-	bool_t use_authentication = false;
-
-	/* if only username xor password are given, return an error */
-	if ((username != NULL) ^ (password != NULL)) {
-		printf("## error: please pass username _and_ password to %s()!\n",
-			__func__);
-		return -1;
-	}
-
-	if (username != NULL && password != NULL)
-		use_authentication = true;
-
-	if (use_authentication == true) {
-		auth_data.username = username;
-		auth_data.password = password;
-	}
+	auth_data.username = username;
+	auth_data.password = password;
 
 	/* parse the uri_string and return a uri struct */
 	ne_uri uri;
 	if (ne_uri_parse(uri_string, &uri)) {
-		printf("## ne_uri_parse() error: invalid URI '%s'.\n", uri_string);
+		fprintf(stderr,
+			"## ne_uri_parse() error: invalid URI '%s'.\n", uri_string);
 		ne_uri_free(&uri);
 		return -1;
 	}
@@ -173,42 +220,54 @@ int setup_webdav_session(
 			ne_ssl_trust_default_ca(session);
 			ne_ssl_set_verify(session, verify_ssl_certificate, &uri);
 		} else {
-			printf("## error: neon ssl support is not enabled.\n");
+			fprintf(stderr, "## error: neon ssl support is not enabled.\n");
 			ne_session_destroy(session);
 			ne_uri_free(&uri);
 			return -1;
 		}
 	}
 
-	/* authentication */
-	if (use_authentication == true)
-		ne_set_server_auth(session, ne_set_server_auth_callback, NULL);
+	/* enable this for on-demand authentication */
+	ne_set_server_auth(session, ne_set_server_auth_callback, NULL);
 
 	/* enable redirect support */
 	ne_redirect_register(session);
 
-	/* try to access the server */
-	ne_server_capabilities capabilities;
-	int ret = ne_options(session, uri.path, &capabilities);
-	if (ret != NE_OK) {
-		printf("## error: could not mount remote server '%s'. ", uri_string);
-		printf("reason: %s", ne_get_error(session));
-		/* if we got a redirect, print the new destination uri and exit */
-		if (ret == NE_REDIRECT) {
-			const ne_uri *new_uri = ne_redirect_location(session);
-			char *new_uri_string = ne_uri_unparse(new_uri);
-			printf(" to '%s'", new_uri_string);
-			FREE(new_uri_string);
-		}
-		printf(".\n");
+	/* escape the path for the case that it contains special chars */
+	char *path = unify_path(uri.path, ESCAPE);
+	if (path == NULL) {
+		printf("## error: unify_path() returned NULL\n");
 		ne_session_destroy(session);
 		ne_uri_free(&uri);
 		return -1;
 	}
 
+	/* try to access the server */
+	ne_server_capabilities capabilities;
+	int ret = ne_options(session, path, &capabilities);
+	if (ret != NE_OK) {
+		fprintf(stderr,
+			"## error: could not mount remote server '%s'. ", uri_string);
+		fprintf(stderr, "reason: %s", ne_get_error(session));
+		/* if we got a redirect, print the new destination uri and exit */
+		if (ret == NE_REDIRECT) {
+			const ne_uri *new_uri = ne_redirect_location(session);
+			char *new_uri_string = ne_uri_unparse(new_uri);
+			fprintf(stderr, " to '%s'", new_uri_string);
+			FREE(new_uri_string);
+		}
+		fprintf(stderr, ".\n");
+		ne_session_destroy(session);
+		ne_uri_free(&uri);
+		FREE(path);
+		return -1;
+	}
+	FREE(path);
+
 	/* is this a webdav server that fulfills webdav class 1? */
 	if (capabilities.dav_class1 != 1) {
-		printf("## error: '%s' is not a webdav enabled server.\n", uri_string);
+		fprintf(stderr, 
+			"## error: '%s' is not a webdav enabled server.\n", uri_string);
 		ne_session_destroy(session);
 		ne_uri_free(&uri);
 		return -1;
@@ -220,6 +279,11 @@ int setup_webdav_session(
 	/* save the remotepath, because each fuse callback method need it to 
 	 * access the files at the webdav server */
 	remotepath_basedir = remove_ending_slash(uri.path);
+	if (remotepath_basedir == NULL) {
+		ne_session_destroy(session);
+		ne_uri_free(&uri);
+		return -1;
+	}
 
 	ne_uri_free(&uri);
 	return 0;
@@ -273,8 +337,8 @@ int lockfile(const char *remotepath, const int timeout)
 
 	/* we already hold a lock for this file, simply return 0 */
 	if (lock != NULL) {
-		if (debug_mode == true)
-			printf("++ file '%s' is already locked.\n", remotepath);
+		if (wdfs.debug == true)
+			fprintf(stderr, "++ file '%s' is already locked.\n", remotepath);
 		return 0;
 	}
 
@@ -289,14 +353,14 @@ int lockfile(const char *remotepath, const int timeout)
 	lock->uri.path = ne_strdup(remotepath);
 
 	if (ne_lock(session, lock)) {
-		printf("## ne_lock() error:\n");
-		printf("## could _not_ lock file '%s'.\n", lock->uri.path);
+		fprintf(stderr, "## ne_lock() error:\n");
+		fprintf(stderr, "## could _not_ lock file '%s'.\n", lock->uri.path);
 		ne_lock_destroy(lock);
 		return 1;
 	} else {
 		ne_lockstore_add(store, lock);
-		if (debug_mode == true)
-			printf("++ locked file '%s'.\n", remotepath);
+		if (wdfs.debug == true)
+			fprintf(stderr, "++ locked file '%s'.\n", remotepath);
 	}
 
 	return 0;
@@ -317,16 +381,16 @@ int unlockfile(const char *remotepath)
 
 	/* if the lock was found, unlock the file */
 	if (ne_unlock(session, lock)) {
-		printf("## ne_unlock() error:\n");
-		printf("## could _not_ unlock file '%s'.\n", lock->uri.path);
+		fprintf(stderr, "## ne_unlock() error:\n");
+		fprintf(stderr, "## could _not_ unlock file '%s'.\n", lock->uri.path);
 		ne_lock_destroy(lock);
 		return 1;
 	} else {
 		/* on success remove the lock from the store and destroy the lock */
 		ne_lockstore_remove(store, lock);
 		ne_lock_destroy(lock);
-		if (debug_mode == true)
-			printf("++ unlocked file '%s'.\n", remotepath);
+		if (wdfs.debug == true)
+			fprintf(stderr, "++ unlocked file '%s'.\n", remotepath);
 	}
 
 	return 0;
@@ -343,19 +407,21 @@ void unlock_all_files()
 		this_lock = ne_lockstore_first(store);
 		while (this_lock != NULL) {
 			if (ne_unlock(session, this_lock)) {
-				printf("## ne_unlock() error:\n");
-				printf("## could _not_ unlock file '%s'.\n", this_lock->uri.path);
+				fprintf(stderr,
+					"## ne_unlock() error:\n"
+				 	"## could _not_ unlock file '%s'.\n", this_lock->uri.path);
 			} else {
-				if (debug_mode == true)
-					printf("++ unlocked file '%s'.\n", this_lock->uri.path);
+				if (wdfs.debug == true)
+					fprintf(stderr,
+						"++ unlocked file '%s'.\n", this_lock->uri.path);
 			}
 			/* get the next lock from the lockstore */
 			this_lock = ne_lockstore_next(store);
 		}
 
 		/* finally destroy the lockstore */
-		if (debug_mode == true)
-			printf("++ destroying lockstore.\n");
+		if (wdfs.debug == true)
+			fprintf(stderr, "++ destroying lockstore.\n");
 		ne_lockstore_destroy(store);
 	}
 }
